package sk.upjs.ics.kniznica.db;

import java.sql.Connection;
import java.sql.Driver;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;
import java.util.UUID;
import org.hsqldb.jdbc.JDBCDriver;
import sk.upjs.ics.kniznica.Kniha;
import sk.upjs.ics.kniznica.RiadkovyPosluchac;

public class Databaza {
    static {
        new JDBCDriver();
    }

    public void vytvorTabulky() {
        String sql = "CREATE TABLE kniha ("
                    + "ID INTEGER NOT NULL PRIMARY KEY,"
                    + "autor VARCHAR(200),"
                    + "nazov VARCHAR(200))";
        vykonajInsertAleboUpdate(sql);
    }

    public void vykonajSql(String sql, RiadkovyPosluchac posluchac) {
        Connection con = null;
        Statement prikaz = null;
        ResultSet rs = null;
        try {
            con = DriverManager.getConnection("jdbc:hsqldb:hsql://localhost/xdb", "SA", "");
            prikaz = con.createStatement();

            rs = prikaz.executeQuery(sql);


            while(rs.next()) {
                posluchac.nastalRiadok(rs);
            }


        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            try {
                if(rs != null) {
                    rs.close();
                }
            } catch (SQLException ex) {
                ex.printStackTrace();
            }

            try {
                if(prikaz != null) {
                    prikaz.close();
                }
            } catch (SQLException ex) {
                ex.printStackTrace();
            }
            try {
                if(con != null) {
                    con.close();
                }
            } catch (SQLException ex) {
                ex.printStackTrace();
            }
        }
    }

    public List<Kniha> vyhladajPodlaNazvu(String nazov) {
        List<Kniha> najdeneKnihy = new LinkedList<Kniha>();

        Connection con = null;
        Statement prikaz = null;
        ResultSet rs = null;
        try {
            con = DriverManager.getConnection("jdbc:hsqldb:hsql://localhost/xdb", "SA", "");
            prikaz = con.createStatement();

            rs = prikaz.executeQuery("SELECT * FROM kniha WHERE nazov = '" + nazov + "'");


            while(rs.next()) {
                Kniha kniha = new Kniha();
                kniha.setId(rs.getInt("id"));
                kniha.setAutor(rs.getString("autor"));
                kniha.setNazov(rs.getString("nazov"));

                najdeneKnihy.add(kniha);
            }


        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            try {
                if(rs != null) {
                    rs.close();
                }
            } catch (SQLException ex) {
                ex.printStackTrace();
            }

            try {
                if(prikaz != null) {
                    prikaz.close();
                }
            } catch (SQLException ex) {
                ex.printStackTrace();
            }
            try {
                if(con != null) {
                    con.close();
                }
            } catch (SQLException ex) {
                ex.printStackTrace();
            }
        }
        return najdeneKnihy;
        
    }

    public Kniha najdiPodlaId(int id) {
        Connection con = null;
        Statement prikaz = null;
        ResultSet rs = null;
        try {
            con = DriverManager.getConnection("jdbc:hsqldb:hsql://localhost/xdb", "SA", "");
            prikaz = con.createStatement();

            rs = prikaz.executeQuery("SELECT * FROM kniha WHERE id = " + id);
            if(!rs.next()) {
                return null;
            }

            Kniha kniha = new Kniha();
            kniha.setId(id);
            kniha.setAutor(rs.getString("autor"));
            kniha.setNazov(rs.getString("nazov"));

            return kniha;

        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            try {
                if(rs != null) {
                    rs.close();
                }
            } catch (SQLException ex) {
                ex.printStackTrace();
            }

            try {
                if(prikaz != null) {
                    prikaz.close();
                }
            } catch (SQLException ex) {
                ex.printStackTrace();
            }
            try {
                if(con != null) {
                    con.close();
                }
            } catch (SQLException ex) {
                ex.printStackTrace();
            }
        }
        return null;
    }

    private void vykonajInsertAleboUpdate(String sql) {
        System.out.println(sql);

        Connection con = null;
        Statement prikaz = null;
        try {
            con = DriverManager.getConnection("jdbc:hsqldb:hsql://localhost/xdb", "SA", "");
            prikaz = con.createStatement();

            prikaz.executeUpdate(sql);
        } catch (SQLException e) {
            e.printStackTrace();
        } finally {
            try {
                if(prikaz != null) {
                    prikaz.close();
                }
            } catch (SQLException ex) {
                ex.printStackTrace();
            }
            try {
                if(con != null) {
                    con.close();
                }
            } catch (SQLException ex) {
                ex.printStackTrace();
            }
        }
    }

    public void zatvor() {
        vykonajInsertAleboUpdate("SHUTDOWN");
    }

    public void vlozData(Kniha kniha) {
        vykonajInsertAleboUpdate("INSERT INTO KNIHA VALUES ("
                + kniha.getId()
                + ", '" + kniha.getAutor() + "'"
                + ", '" + kniha.getNazov() + "'"
                + ")");
    }

    public static void main(String[] args) {
        Databaza db = new Databaza();
        /*

        db.vytvorTabulky();
        db.zatvor();
        */
/*
        Kniha k = new Kniha();
        k.setId(25);
        k.setAutor("J. K. Rowling");
        k.setNazov("HP 7");
        db.vlozData(k);

       Kniha k = db.najdiPodlaId(25);
       System.out.println(k);
*/

        RiadkovyPosluchac p = new ZistiPocetPosluchac();
        db.vykonajSql("SELECT COUNT(*) FROM KNIHA", p);

        KnihaPosluchac p2 = new KnihaPosluchac();
        db.vykonajSql("SELECT * FROM KNIHA WHERE ID > 2", p2);

        System.out.println(p2.getKnihy());
    }


    public static class KnihaPosluchac implements RiadkovyPosluchac {
        private List<Kniha> knihy = new LinkedList<Kniha>();

        public void nastalRiadok(ResultSet resultSet) throws SQLException {
            Kniha k = new Kniha();
            k.setId(resultSet.getInt("id"));
            k.setNazov(resultSet.getString("nazov"));
            k.setAutor(resultSet.getString("autor"));

            knihy.add(k);
        }

        public List<Kniha> getKnihy() {
            return knihy;
        }

        

    }

    public static class ZistiPocetPosluchac implements RiadkovyPosluchac {
        public void nastalRiadok(ResultSet resultSet) throws SQLException {
            int pocetKnih = resultSet.getInt(1);
            System.out.println("Pocet knih: " + pocetKnih);
        }
    }

}
